@@ -16,8 +16,8 @@ from sklearn.linear_model import LogisticRegression
 from sklearn.utils import check_X_y
 
 from alipy.query_strategy.query_labels import QueryInstanceQBC, \
-         QueryInstanceUncertainty, QueryInstanceRandom, QureyExpectedErrorReduction, QueryInstanceQUIRE, \
-          QueryInstanceGraphDensity, QueryInstanceBMDR, QueryInstanceSPAL, QueryInstanceLAL
+    QueryInstanceUncertainty, QueryInstanceRandom, QueryInstanceQUIRE, \
+    QueryInstanceGraphDensity, QueryInstanceBMDR, QueryInstanceSPAL, QueryInstanceLAL, QueryExpectedErrorReduction
 from ..data_manipulate.al_split import split
 from .experiment_analyser import ExperimentAnalyser
 from .state import State
@@ -27,6 +27,9 @@ from ..index.index_collections import IndexCollection
 from ..utils.multi_thread import aceThreading
 from ..metrics import performance
 
+__all__ = ['AlExperiment',
+           ]
+
 
 class AlExperiment:
     """AlExperiment is a  class to encapsulate various tools
@@ -64,6 +67,9 @@ class AlExperiment:
         'percent_of_unlabel': stop when specific percentage of unlabeled data pool is labeled.
         'time_limit': stop when CPU time reaches the limit.
 
+    stopping_value: {int, float}, optional (default=None)
+        The value of the corresponding stopping criterion.
+
     batch_size: int, optional (default=1)
         batch size of AL
 
@@ -147,8 +153,9 @@ class AlExperiment:
         else:
             # a pre-defined strategy in ALiPy
             if strategy not in ['QueryInstanceQBC', 'QueryInstanceUncertainty', 'QueryRandom', 'QueryInstanceRandom',
-                                'QureyExpectedErrorReduction', 'QueryInstanceGraphDensity', 'QueryInstanceQUIRE',
-                                'QueryInstanceBMDR', 'QueryInstanceSPAL', 'QueryInstanceLAL']:
+                                'QueryInstanceGraphDensity', 'QueryInstanceQUIRE',
+                                'QueryInstanceBMDR', 'QueryInstanceSPAL', 'QueryInstanceLAL',
+                                'QueryExpectedErrorReduction']:
                 raise NotImplementedError('Strategy {} is not implemented. Specify a valid '
                                           'method name or privide a callable object.'.format(str(strategy)))
             else:
@@ -162,8 +169,8 @@ class AlExperiment:
                     self._query_function = QueryInstanceUncertainty(self._X, self._y, measure)
                 elif strategy == 'QueryInstanceRandom' or strategy == 'QueryRandom':
                     self._query_function = QueryInstanceRandom(self._X, self._y)
-                elif strategy == 'QureyExpectedErrorReduction':
-                    self._query_function = QureyExpectedErrorReduction(self._X, self._y)
+                elif strategy == 'QueryExpectedErrorReduction':
+                    self._query_function = QueryExpectedErrorReduction(self._X, self._y)
                 elif strategy == 'QueryInstanceGraphDensity' or strategy == 'QueryInstanceQUIRE':
                     if self._train_idx is None:
                         raise ValueError(
@@ -176,23 +183,23 @@ class AlExperiment:
                     gamma = kwargs.pop('gamma', 0.1)
                     rho = kwargs.pop('rho', 1)
                     self._query_function = QueryInstanceBMDR(self._X, self._y, beta, gamma, rho, **kwargs)
-                    self.qp_solver = kwargs.pop('qp_sover', 'ECOS')                      
+                    self.qp_solver = kwargs.pop('qp_sover', 'ECOS')
                 elif strategy == 'QueryInstanceSPAL':
-                    mu = kwargs.pop('mu',0.1)
-                    gamma = kwargs.pop('gamma',0.1)
-                    rho = kwargs.pop('rho',1)
-                    lambda_init = kwargs.pop('lambda_init',0.1)
-                    lambda_pace = kwargs.pop('lambda_pace',0.01)
-                    self._query_function = QueryInstanceSPAL(self._X, self._y, mu, gamma, rho, lambda_init, lambda_pace, **kwargs)
+                    mu = kwargs.pop('mu', 0.1)
+                    gamma = kwargs.pop('gamma', 0.1)
+                    rho = kwargs.pop('rho', 1)
+                    lambda_init = kwargs.pop('lambda_init', 0.1)
+                    lambda_pace = kwargs.pop('lambda_pace', 0.01)
+                    self._query_function = QueryInstanceSPAL(self._X, self._y, mu, gamma, rho, lambda_init, lambda_pace,
+                                                             **kwargs)
                     self.qp_solver = kwargs.pop('qp_sover', 'ECOS')
                 elif strategy == 'QueryInstanceLAL':
                     mode = kwargs.pop('mode', 'LAL_iterative')
                     data_path = kwargs.pop('data_path', '.')
                     cls_est = kwargs.pop('cls_est', 50)
                     train_slt = kwargs.pop('train_slt', True)
-                    self._query_function = QueryInstanceLAL(self._X, self._y, mode, data_path, cls_est, train_slt, **kwargs)
-
-
+                    self._query_function = QueryInstanceLAL(self._X, self._y, mode, data_path, cls_est, train_slt,
+                                                            **kwargs)
 
     def set_performance_metric(self, performance_metric='accuracy_score', **kwargs):
         """
@@ -372,8 +379,6 @@ class AlExperiment:
             elif self._query_function_name == 'QueryInstanceQUIRE':
                 querfunction = QueryInstanceQUIRE(self._X, self._y, train_id, **self._query_function_kwargs)
 
-        
-
         # performance calc
         perf_result = self._performance_metric(pred, self._y[test_id])
 
@@ -392,7 +397,8 @@ class AlExperiment:
                     select_ind = self._query_function.select(Lcollection, Ucollection, batch_size=self._batch_size,
                                                              model=self._model)
                 elif self._query_function_name == 'QueryInstanceBMDR' or self._query_function_name == 'QueryInstanceSPAL':
-                    select_ind = self._query_function.select(Lcollection, Ucollection, batch_size=self._batch_size, qp_solver=self.qp_solver)
+                    select_ind = self._query_function.select(Lcollection, Ucollection, batch_size=self._batch_size,
+                                                             qp_solver=self.qp_solver)
                 else:
                     select_ind = self._query_function.select(Lcollection, Ucollection, batch_size=self._batch_size)
             else:
@@ -442,4 +448,4 @@ class AlExperiment:
         ea = ExperimentAnalyser()
         ea.add_method(self._query_function_name, self._experiment_result)
         print(ea)
-        ea.plot_learning_curves(title=title)
\ No newline at end of file
+        ea.plot_learning_curves(title=title)
