@@ -25,8 +25,16 @@ from sklearn.linear_model import LogisticRegression
 from ..index import IndexCollection, MultiLabelIndexCollection
 from ..index.multi_label_tools import get_Xy_in_multilabel
 from ..utils.misc import randperm
+from ..utils.ace_warnings import *
 from .base import BaseIndexQuery, BaseMultiLabelQuery
 
+__all__ = ['LabelRankingModel',
+           'QueryMultiLabelQUIRE',
+           'QueryMultiLabelAUDI',
+           'QueryMultiLabelMMC',
+           'QueryMultiLabelAdaptive',
+           'QueryMultiLabelRandom',
+           ]
 
 class _LabelRankingModel_MatlabVer:
     """Label ranking model is a classification model in multi-label setting.
@@ -51,7 +59,7 @@ class _LabelRankingModel_MatlabVer:
     [1] S.-J. Huang and Z.-H. Zhou. Active query driven by uncertainty and
         diversity for incremental multi-label learning. In Proceedings
         of the 13th IEEE International Conference on Data Mining, pages
-        1079–1084, Dallas, TX, 2013.
+        1079-1084, Dallas, TX, 2013.
     """
 
     def __init__(self, init_X=None, init_y=None):
@@ -69,7 +77,8 @@ class _LabelRankingModel_MatlabVer:
             self._init_flag = True
 
     def get_BV(self, AB, AV, Anum):
-        return (AV / Anum).T.dot(AB / Anum)
+        bv = (AV / Anum).T.dot(AB / Anum)
+        return bv
 
     def init_model_train(self, init_data=None, init_targets=None, n_repeat=10):
         if init_data is None:
@@ -218,11 +227,11 @@ class _LabelRankingModel_MatlabVer:
                 idx_down = np.nonzero(norms > norm_up)[0]
                 B[:, idx_class * num_sub + idx_max_class] = tmp1
                 B[:, idx_pick * num_sub + idx_max_pick] = tmp2
-                if idx_down:
+                if idx_down.size > 0:
                     norms = norms[norms > norm_up]
                     for k in range(len(idx_down)):
                         V[:, idx_down[k]] = V[:, idx_down[k]] * norm_up / norms[k]
-            if trounds > average_begin and i % average_size == 0:
+            if i == 0 or (trounds > average_begin and i % average_size == 0):
                 AB = AB + B
                 AV = AV + V
                 Anum = Anum + 1
@@ -265,6 +274,11 @@ class LabelRankingModel(_LabelRankingModel_MatlabVer):
 
     This class is mainly used for AURO and AUDI method for multi label querying.
 
+    !! IMPORTANT
+    1. This model is scaling sensitive. If you find the optimization process is not converge,
+    (e.g., ZeroDivisionError: division by zero, RuntimeWarning: overflow encountered in multiply,
+    or the values of BV tend to infinite.) please try to normalize your data first.
+
     Parameters
     ----------
     init_X: 2D array, optional (default=None)
@@ -281,18 +295,20 @@ class LabelRankingModel(_LabelRankingModel_MatlabVer):
     [1] S.-J. Huang and Z.-H. Zhou. Active query driven by uncertainty and
         diversity for incremental multi-label learning. In Proceedings
         of the 13th IEEE International Conference on Data Mining, pages
-        1079–1084, Dallas, TX, 2013.
+        1079-1084, Dallas, TX, 2013.
     """
 
     def __init__(self, init_X=None, init_y=None, **kwargs):
         super(LabelRankingModel, self).__init__(init_X, init_y)
+        self._ini_parameters = None
         if self._init_flag is True:
             n_repeat = kwargs.pop('n_repeat', 10)
+            self._ini_parameters = self.init_model_train(self._init_X, self._init_y, n_repeat=n_repeat)
             self._B, self._V, self._AB, self._AV, self._Anum, self._trounds, self._costs, self._norm_up, \
             self._step_size0, self._num_sub, self._lmbda, self._avg_begin, self._avg_size, self._n_repeat, \
-            self._max_query = self.init_model_train(self._init_X, self._init_y, n_repeat=n_repeat)
+            self._max_query = self._ini_parameters
 
-    def fit(self, X, y, n_repeat=10):
+    def fit(self, X, y, n_repeat=10, is_incremental=False):
         """Train the model from X and y.
 
         Parameters
@@ -305,18 +321,27 @@ class LabelRankingModel(_LabelRankingModel_MatlabVer):
 
         n_repeat: int, optional (default=10)
             The number of optimization iterations.
+
+        is_incremental: bool, optional (default=False)
+            Whether to train the model in an incremental way.
         """
-        if self._init_flag is False:
+        if not self._init_flag or not self._ini_parameters:
+            self._ini_parameters = self.init_model_train(X, y, n_repeat=n_repeat)
             self._B, self._V, self._AB, self._AV, self._Anum, self._trounds, self._costs, self._norm_up, \
             self._step_size0, self._num_sub, self._lmbda, self._avg_begin, self._avg_size, self._n_repeat, \
-            self._max_query = self.init_model_train(X, y, n_repeat=n_repeat)
+            self._max_query = self._ini_parameters
             self._init_flag = True
-        else:
-            for i in range(n_repeat):
-                self._B, self._V, self._AB, self._AV, self._Anum, self._trounds = self.train_model(
-                    X, y, self._B, self._V, self._costs, self._norm_up,
-                    self._step_size0, self._num_sub, self._AB, self._AV, self._Anum, self._trounds,
-                    self._lmbda, self._avg_begin, self._avg_size)
+
+        if not is_incremental:
+            self._B, self._V, self._AB, self._AV, self._Anum, self._trounds, self._costs, self._norm_up, \
+            self._step_size0, self._num_sub, self._lmbda, self._avg_begin, self._avg_size, self._n_repeat, \
+            self._max_query = self._ini_parameters
+
+        for i in range(n_repeat):
+            self._B, self._V, self._AB, self._AV, self._Anum, self._trounds = self.train_model(
+                X, y, self._B, self._V, self._costs, self._norm_up,
+                self._step_size0, self._num_sub, self._AB, self._AV, self._Anum, self._trounds,
+                self._lmbda, self._avg_begin, self._avg_size)
 
     def predict(self, X):
         BV = self.get_BV(self._AB, self._AV, self._Anum)
@@ -372,7 +397,7 @@ class QueryMultiLabelQUIRE(BaseMultiLabelQuery):
     [1] Huang, S.; Jin, R.; and Zhou, Z. 2014. Active learning by
         querying informative and representative examples. IEEE
         Transactions on Pattern Analysis and Machine Intelligence
-        36(10):1936–1949
+        36(10):1936-1949
     """
 
     def __init__(self, X, y, **kwargs):
@@ -497,7 +522,7 @@ class QueryMultiLabelAUDI(BaseMultiLabelQuery):
     [1] S.-J. Huang and Z.-H. Zhou. Active query driven by uncertainty and
         diversity for incremental multi-label learning. In Proceedings
         of the 13th IEEE International Conference on Data Mining, pages
-        1079–1084, Dallas, TX, 2013.
+        1079-1084, Dallas, TX, 2013.
     """
 
     def __init__(self, X, y):
@@ -506,7 +531,7 @@ class QueryMultiLabelAUDI(BaseMultiLabelQuery):
         #     self.y = np.hstack((self.y, 2 * np.ones((self.y.shape[0], 1))))
         self._lr_model = LabelRankingModel()
 
-    def select(self, label_index, unlabel_index, epsilon=0.5, **kwargs):
+    def select(self, label_index, unlabel_index, model=None, epsilon=0.5, **kwargs):
         """Select a subset from the unlabeled set, return the selected instance and label.
 
         Parameters
@@ -521,6 +546,10 @@ class QueryMultiLabelAUDI(BaseMultiLabelQuery):
             MultiLabelIndexCollection or a list of tuples with 2 elements, in which,
             the 1st element is the index of instance and the 2nd element is the index of labels.
 
+        model: LabelRankingModel (optional, default=None)
+            A trained model used to predict unlabeled data.
+            If None is passed, it will re-train a LabelRanking model.
+
         epsilon: float, optional (default=0.5)
             The threshold to avoid zero-division.
 
@@ -534,11 +563,20 @@ class QueryMultiLabelAUDI(BaseMultiLabelQuery):
         unlabel_index = self._check_multi_label_ind(unlabel_index)
         label_index = self._check_multi_label_ind(label_index)
 
-        # select instance by LCI
-        W = unlabel_index.get_matrix_mask(mat_shape=self.y.shape, fill_value=1, sparse=False)
         unlab_data, _, data_ind = get_Xy_in_multilabel(index=unlabel_index, X=self.X, y=self.y)
-        lab_data, lab_lab, _ = get_Xy_in_multilabel(index=label_index, X=self.X, y=self.y)
-        self._lr_model.fit(lab_data, lab_lab)
+        W = unlabel_index.get_matrix_mask(mat_shape=self.y.shape, fill_value=1, sparse=False)
+        if model is not None:
+            assert isinstance(model, LabelRankingModel), 'Model for selection must be LabelRanking model in ' \
+                                                         'AUDI algorithm. Try to pass model=None to use the ' \
+                                                         'default model'
+            self._lr_model = model
+            if not self._lr_model._init_flag:   # not trained
+                lab_data, lab_lab, _ = get_Xy_in_multilabel(index=label_index, X=self.X, y=self.y)
+                self._lr_model.fit(lab_data, lab_lab)
+        else:
+            # select instance by LCI
+            lab_data, lab_lab, _ = get_Xy_in_multilabel(index=label_index, X=self.X, y=self.y)
+            self._lr_model.fit(lab_data, lab_lab)
         pres, labels = self._lr_model.predict(unlab_data)
         avgP = np.mean(np.sum(self.y[label_index.get_unbroken_instances(), :] == 1, axis=1))
         insvals = -np.abs((np.sum(labels == 1, axis=1) - avgP) / np.fmax(np.sum(W[data_ind, :] == 1, axis=1), epsilon))
@@ -552,10 +590,18 @@ class QueryMultiLabelAUDI(BaseMultiLabelQuery):
         pres_tmp = pres[:, 0:-1]
         pres_tmp[pres_mask] = np.NINF
         pres[:, 0:-1] = pres_tmp
+        sel_ins_label_mask = pres_mask[selected_ins]
 
         dis = np.abs(pres[selected_ins, 0:-1] - pres[selected_ins, -1])
-        selected_ins = data_ind[selected_ins]
         selected_lab = np.argmin(dis)
+        if sel_ins_label_mask[selected_lab]:
+            # select a labeled entry
+            argsorted_dis = np.argsort(dis) # ascent
+            for dis_ind in argsorted_dis:
+                if not sel_ins_label_mask[dis_ind]:
+                    selected_lab = dis_ind
+                    break
+        selected_ins = data_ind[selected_ins]
 
         return [(selected_ins, selected_lab)]
 
@@ -758,13 +804,16 @@ class QueryMultiLabelMMC(BaseIndexQuery):
     y: array-like
         Label matrix of the whole dataset. It is a reference which will not use additional memory.
 
-    base_learner :  object instance
-        The base learner for binary relavance, should support predict_proba
-
     br_base : ProbabilisticModel object instance
         The base learner for the binary relevance in MMC.
         Should support predict_proba.
 
+        .. Deprecated
+
+    base_clf : ProbabilisticModel object instance
+        The base learner for the binary relevance in MMC.
+        Should support predict_proba.
+
     logreg_param : dict, optional (default={})
         Setting the parameter for the logistic regression that are used to
         predict the number of labels for a given feature vector. Parameter
@@ -795,9 +844,15 @@ class QueryMultiLabelMMC(BaseIndexQuery):
                                         'random_state': random_state})
         self.logistic_regression_ = LogisticRegression(**self.logreg_param)
 
-        self.br_base = kwargs.pop('br_base',
-                                  SVC(kernel='linear', probability=True,
-                                      random_state=random_state))
+        br_base = kwargs.pop('br_base', None)
+        if br_base is not None:
+            warnings.warn('br_base is deprecated, please use base_clf instead.', DeprecationWarning)
+        br_base2 = kwargs.pop('base_clf', None)
+        if br_base is None and br_base2 is None:
+            self.br_base = SVC(kernel='linear', probability=True,
+                               random_state=random_state)
+        else:
+            self.br_base = br_base2 if br_base is None else br_base
     
     def sequential_select(self, label_index, unlabel_index):
         """
@@ -866,7 +921,7 @@ class QueryMultiLabelMMC(BaseIndexQuery):
 
         return unlabel_index[ask_id]
         
-    def select(self, label_index, unlabel_index, batch_size=1):
+    def select(self, label_index, unlabel_index, batch_size=1, **kwargs):
         """
             Select the unlabel data in batch mode.
         Parameters
@@ -1037,11 +1092,13 @@ class QueryMultiLabelAdaptive(BaseIndexQuery):
             pos = np.copy(br_real)
             pos[br_real < 0] = 1
             pos = np.max((1. - pos), axis=1)
+            pos = np.array([0 if i<0 else i for i in pos])
 
             neg = np.copy(br_real)
             neg[br_real > 0] = -1
             neg = np.max((1. + neg), axis=1)
-
+            neg = np.array([0 if i<0 else i for i in neg])
+            
             err = neg + pos
 
             approx_err.append(np.sum(err))
@@ -1051,7 +1108,7 @@ class QueryMultiLabelAdaptive(BaseIndexQuery):
 
         return unlabel_index[ask_idx]
 
-    def select(self, label_index, unlabel_index, batch_size=1):
+    def select(self, label_index, unlabel_index, batch_size=1, **kwargs):
         """
             Select the unlabel data in batch mode.
         Parameters
