@@ -27,6 +27,20 @@ from .base import BaseIndexQuery
 from ..utils.ace_warnings import *
 from ..utils.misc import nsmallestarg, randperm, nlargestarg
 
+__all__ = ['QueryInstanceUncertainty',
+           'QueryRandom',
+           'QueryInstanceRandom',
+           'QueryInstanceQBC',
+           'QueryExpectedErrorReduction',
+           'QueryInstanceQUIRE',
+           'QueryInstanceGraphDensity',
+           'QueryInstanceBMDR',
+           'QueryInstanceSPAL',
+           'QueryInstanceLAL',
+           'QueryInstanceCoresetGreedy',
+           'QueryInstanceDensityWeighted',
+           ]
+
 
 def _get_proba_pred(unlabel_x, model):
     """Get the probabilistic prediction results of the unlabeled set.
@@ -201,7 +215,7 @@ class QueryInstanceUncertainty(BaseIndexQuery):
         if self.measure == 'entropy':
             # calc entropy
             pv[pv <= 0] = 1e-06  # avoid zero division
-            entro = [-np.sum(vec * np.log(vec)) for vec in pv]
+            entro = [-np.sum(vec * np.log(vec+1e-9)) for vec in pv]
             assert (len(np.shape(entro)) == 1)
             return unlabel_index[nlargestarg(entro, batch_size)]
 
@@ -237,7 +251,7 @@ class QueryInstanceUncertainty(BaseIndexQuery):
             raise Exception('2d array with the shape [n_samples, n_classes]'
                             ' is expected, but received shape: \n%s' % str(spv))
         # calc entropy
-        entropy = [-np.sum(vec * np.log(vec)) for vec in pv]
+        entropy = [-np.sum(vec * np.log(vec+1e-9)) for vec in pv]
         return entropy
 
 
@@ -247,6 +261,10 @@ class QueryRandom(BaseIndexQuery):
     The random strategy has been re-named to QueryInstanceRandom,
     this class will be deleted in v1.0.5.
     """
+    def __init__(self, X=None, y=None):
+        warnings.warn("QueryRandom will be deleted in the future. Use QueryInstanceRandom instead.",
+                      category=DeprecationWarning)
+        super(QueryRandom, self).__init__(X, y)
 
     def select(self, label_index, unlabel_index, batch_size=1, **kwargs):
         """Select indexes randomly.
@@ -341,11 +359,11 @@ class QueryInstanceQBC(BaseIndexQuery):
     ----------
     [1] H.S. Seung, M. Opper, and H. Sompolinsky. Query by committee.
         In Proceedings of the ACM Workshop on Computational Learning Theory,
-        pages 287–294, 1992.
+        pages 287-294, 1992.
 
     [2] N. Abe and H. Mamitsuka. Query learning strategies using boosting and bagging.
         In Proceedings of the International Conference on Machine Learning (ICML),
-        pages 1–9. Morgan Kaufmann, 1998.
+        pages 1-9. Morgan Kaufmann, 1998.
     """
 
     def __init__(self, X=None, y=None, method='query_by_bagging', disagreement='vote_entropy'):
@@ -504,7 +522,7 @@ class QueryInstanceQBC(BaseIndexQuery):
         ----------
         [1] I. Dagan and S. Engelson. Committee-based sampling for training probabilistic
             classifiers. In Proceedings of the International Conference on Machine
-            Learning (ICML), pages 150–157. Morgan Kaufmann, 1995.
+            Learning (ICML), pages 150-157. Morgan Kaufmann, 1995.
         """
         score = []
         input_shape, committee_size = cls()._check_committee_results(predict_matrices)
@@ -520,7 +538,7 @@ class QueryInstanceQBC(BaseIndexQuery):
                 # calc each label
                 for vote in voting:
                     if vote != 0:
-                        tmp += vote / len(predict_matrices) * np.log(vote / len(predict_matrices))
+                        tmp += vote / len(predict_matrices) * np.log((vote+1e-9) / len(predict_matrices))
                 score.append(-tmp)
         else:
             input_mat = np.array([X for X in predict_matrices if X is not None])
@@ -530,7 +548,7 @@ class QueryInstanceQBC(BaseIndexQuery):
                 count_dict = collections.Counter(input_mat[:, i])
                 tmp = 0
                 for key in count_dict:
-                    tmp += count_dict[key] / committee_size * np.log(count_dict[key] / committee_size)
+                    tmp += count_dict[key] / committee_size * np.log((count_dict[key]+1e-9) / committee_size)
                 score.append(-tmp)
         return score
 
@@ -555,7 +573,7 @@ class QueryInstanceQBC(BaseIndexQuery):
         ----------
         [1] A. McCallum and K. Nigam. Employing EM in pool-based active learning for
             text classification. In Proceedings of the International Conference on Machine
-            Learning (ICML), pages 359–367. Morgan Kaufmann, 1998.
+            Learning (ICML), pages 359-367. Morgan Kaufmann, 1998.
         """
         score = []
         input_shape, committee_size = cls()._check_committee_results(predict_matrices)
@@ -567,9 +585,9 @@ class QueryInstanceQBC(BaseIndexQuery):
                 tmp = 0
                 # calc each label
                 for lab in range(label_num):
-                    committee_consensus = np.sum(instance_mat[:, lab]) / committee_size
+                    committee_consensus = np.sum(instance_mat[:, lab]) / committee_size + 1e-9
                     for committee in range(committee_size):
-                        tmp += instance_mat[committee, lab] * np.log(instance_mat[committee, lab] / committee_consensus)
+                        tmp += instance_mat[committee, lab] * np.log((instance_mat[committee, lab]+1e-9) / committee_consensus)
                 score.append(tmp)
         else:
             raise Exception(
@@ -577,7 +595,7 @@ class QueryInstanceQBC(BaseIndexQuery):
         return score
 
 
-class QureyExpectedErrorReduction(BaseIndexQuery):
+class QueryExpectedErrorReduction(BaseIndexQuery):
     """The Expected Error Reduction (ERR) algorithm.
 
     The idea is to estimate the expected future error of a model trained using label set and <x, y> on
@@ -586,7 +604,7 @@ class QureyExpectedErrorReduction(BaseIndexQuery):
     with minimal expected future error (sometimes called risk)
 
     This algorithm needs to re-train the model for multiple times.
-    So There are 2 contraints to the given model.
+    So There are 2 constraints to the given model.
     1. It is a sklearn model (or a model who implements their api).
     2. It has the probabilistic output function predict_proba.
 
@@ -606,12 +624,12 @@ class QureyExpectedErrorReduction(BaseIndexQuery):
     ----------
     [1] N. Roy and A. McCallum. Toward optimal active learning through sampling
         estimation of error reduction. In Proceedings of the International Conference on
-        Machine Learning (ICML), pages 441–448. Morgan Kaufmann, 2001.
+        Machine Learning (ICML), pages 441-448. Morgan Kaufmann, 2001.
 
     """
 
-    def __init__(self, X=None, y=None):
-        super(QureyExpectedErrorReduction, self).__init__(X, y)
+    def __init__(self, X, y):
+        super(QueryExpectedErrorReduction, self).__init__(X, y)
 
     def log_loss(self, prob):
         """Compute expected log-loss.
@@ -665,7 +683,7 @@ class QureyExpectedErrorReduction(BaseIndexQuery):
 
         # get unlabel_x
         if self.X is None or self.y is None:
-            raise Exception('Data matrix is not provided, use select_by_prediction_mat() instead.')
+            raise Exception('Data matrix is not provided.')
         if model is None:
             model = LogisticRegression(solver='liblinear')
             model.fit(self.X[label_index if isinstance(label_index, (list, np.ndarray)) else label_index.index],
@@ -756,7 +774,7 @@ class QueryInstanceQUIRE(BaseIndexQuery):
     [2] Huang, S.; Jin, R.; and Zhou, Z. 2014. Active learning by
         querying informative and representative examples. IEEE
         Transactions on Pattern Analysis and Machine Intelligence
-        36(10):1936–1949
+        36(10):1936-1949
     """
 
     def __init__(self, X, y, train_idx, **kwargs):
@@ -897,7 +915,7 @@ class QueryInstanceGraphDensity(BaseIndexQuery):
         active learning formulation for object class recognition.
         In 2012 IEEE Conference on Computer Vision and  Pattern Recognition,
         Providence, RI, USA, June 16-21, 2012,
-        3626–3633.
+        3626-3633.
     """
 
     def __init__(self, X, y, train_idx, metric='manhattan'):
@@ -1072,9 +1090,8 @@ class QueryInstanceBMDR(BaseIndexQuery):
                           category=FunctionWarning)
         if len(ul) == 2 and {1, -1} != set(ul):
             y_temp = np.array(copy.deepcopy(self.y))
-            y_temp[y_temp == ul[0]] = 1
-            y_temp[y_temp == ul[1]] = -1
-            self.y = y_temp
+            self.y[y_temp == ul[0]] = 1
+            self.y[y_temp == ul[1]] = -1
 
         self._beta = beta
         self._gamma = gamma
@@ -1180,17 +1197,44 @@ class QueryInstanceBMDR(BaseIndexQuery):
             pred_of_unlab = tau.dot(KLU)
             a = pred_of_unlab * pred_of_unlab + 2 * np.abs(pred_of_unlab)
             q = self._beta * (
-                (U_len - batch_size) / N * np.ones(L_len).dot(KLU) - (L_len + batch_size) / N * np.ones(U_len).dot(
-                    KUU)) + a
+                    (U_len - batch_size) / N * np.ones(L_len).dot(KLU) - (L_len + batch_size) / N * np.ones(U_len).dot(
+                KUU)) + a
 
             # cvx
             x = cvxpy.Variable(U_len)
-            objective = cvxpy.Minimize(0.5 * cvxpy.quad_form(x, P) + q.T * x)
+            objective = cvxpy.Minimize(0.5 * cvxpy.quad_form(x, P) + q.T @ x)
             constraints = [0 <= x, x <= 1, sum(x) == batch_size]
             prob = cvxpy.Problem(objective, constraints)
             # The optimal objective value is returned by `prob.solve()`.
             # print(prob.is_qp())
-            result = prob.solve(solver=cvxpy.OSQP if qp_solver == 'OSQP' else cvxpy.ECOS)
+            try:
+                result = prob.solve(solver=cvxpy.OSQP if qp_solver == 'OSQP' else cvxpy.ECOS)
+            except cvxpy.error.DCPError:
+                # cvx
+                x = cvxpy.Variable(U_len)
+                objective = cvxpy.Minimize(0.5 * cvxpy.quad_form(x, P) + q.T @ x)
+                constraints = [0 <= x, x <= 1]
+                prob = cvxpy.Problem(objective, constraints)
+                # The optimal objective value is returned by `prob.solve()`.
+                try:
+                    result = prob.solve(solver=cvxpy.OSQP if qp_solver == 'OSQP' else cvxpy.ECOS)
+                except cvxpy.error.DCPError:
+                    result = prob.solve(solver=cvxpy.OSQP if qp_solver == 'OSQP' else cvxpy.ECOS, gp=True)
+
+            # Sometimes the constraints can not be satisfied,
+            # thus we relax the constraints to get an approximate solution.
+            if not (type(result) == float and result != float('inf') and result != float('-inf')):
+                # cvx
+                x = cvxpy.Variable(U_len)
+                objective = cvxpy.Minimize(0.5 * cvxpy.quad_form(x, P) + q.T @ x)
+                constraints = [0 <= x, x <= 1]
+                prob = cvxpy.Problem(objective, constraints)
+                # The optimal objective value is returned by `prob.solve()`.
+                try:
+                    result = prob.solve(solver=cvxpy.OSQP if qp_solver == 'OSQP' else cvxpy.ECOS)
+                except cvxpy.error.DCPError:
+                    result = prob.solve(solver=cvxpy.OSQP if qp_solver == 'OSQP' else cvxpy.ECOS, gp=True)
+
             # The optimal value for x is stored in `x.value`.
             # print(x.value)
             dr_weight = np.array(x.value)
@@ -1310,14 +1354,13 @@ class QueryInstanceSPAL(BaseIndexQuery):
         # K: kernel matrix
         super(QueryInstanceSPAL, self).__init__(X, y)
         ul = unique_labels(self.y)
-        if len(unique_labels(self.y)) != 2:
+        if len(ul) != 2:
             warnings.warn("This query strategy is implemented for binary classification only.",
                           category=FunctionWarning)
         if len(ul) == 2 and {1, -1} != set(ul):
             y_temp = np.array(copy.deepcopy(self.y))
-            y_temp[y_temp == ul[0]] = 1
-            y_temp[y_temp == ul[1]] = -1
-            self.y = y_temp
+            self.y[y_temp == ul[0]] = 1
+            self.y[y_temp == ul[1]] = -1
 
         self._mu = mu
         self._gamma = gamma
@@ -1353,16 +1396,19 @@ class QueryInstanceSPAL(BaseIndexQuery):
         pickle_seq = (
             self.X,
             self.y,
-            self._beta,
+            self._mu,
             self._gamma,
             self._rho,
+            self._lambda,
+            self._lambda_init,
+            self._lambda_pace,
             self._kernel,
             self._K
         )
         return pickle_seq
 
     def __setstate__(self, state):
-        self.X, self.y, self._beta, self._gamma, self._rho, self._kernel, self._K = state
+        self.X, self.y, self._mu, self._gamma, self._rho, self._lambda, self._lambda_init, self._lambda_pace, self._kernel, self._K = state
         import cvxpy
         self._cvxpy = cvxpy
 
@@ -1428,31 +1474,38 @@ class QueryInstanceSPAL(BaseIndexQuery):
             pred_of_unlab = theta.dot(KLU)
             a = es_weight * (pred_of_unlab * pred_of_unlab + 2 * np.abs(pred_of_unlab))
             q = self._mu * (
-                (U_len - batch_size) / N * np.ones(L_len).dot(KLU) - (L_len + batch_size) / N * np.ones(U_len).dot(
-                    KUU)) + a
+                    (U_len - batch_size) / N * np.ones(L_len).dot(KLU) - (L_len + batch_size) / N * np.ones(U_len).dot(
+                KUU)) + a
             # cvx
             x = cvxpy.Variable(U_len)
-            objective = cvxpy.Minimize(0.5 * cvxpy.quad_form(x, P) + q.T * x)
-            constraints = [0 <= x, x <= 1, es_weight * x == batch_size]
+            objective = cvxpy.Minimize(0.5 * cvxpy.quad_form(x, P) + q.T @ x)
+            constraints = [0 <= x, x <= 1, es_weight @ x == batch_size]
             prob = cvxpy.Problem(objective, constraints)
             # The optimal objective value is returned by `prob.solve()`.
-            result = prob.solve(solver=cvxpy.OSQP if qp_solver == 'OSQP' else cvxpy.ECOS)
+            # result = prob.solve(solver=cvxpy.OSQP if qp_solver == 'OSQP' else cvxpy.ECOS)
+            try:
+                result = prob.solve(solver=cvxpy.OSQP if qp_solver == 'OSQP' else cvxpy.ECOS)
+            except cvxpy.error.DCPError:
+                result = prob.solve(solver=cvxpy.OSQP if qp_solver == 'OSQP' else cvxpy.ECOS, gp=True)
             # Sometimes the constraints can not be satisfied,
             # thus we relax the constraints to get an approximate solution.
             if not (type(result) == float and result != float('inf') and result != float('-inf')):
-                P = 0.5 * self._mu * KUU
-                pred_of_unlab = theta.dot(KLU)
-                a = es_weight * (pred_of_unlab * pred_of_unlab + 2 * np.abs(pred_of_unlab))
-                q = self._mu * (
-                    (U_len - batch_size) / N * np.ones(L_len).dot(KLU) - (L_len + batch_size) / N * np.ones(U_len).dot(
-                        KUU)) + a
+                # P = 0.5 * self._mu * KUU
+                # pred_of_unlab = theta.dot(KLU)
+                # a = es_weight * (pred_of_unlab * pred_of_unlab + 2 * np.abs(pred_of_unlab))
+                # q = self._mu * (
+                #     (U_len - batch_size) / N * np.ones(L_len).dot(KLU) - (L_len + batch_size) / N * np.ones(U_len).dot(
+                #         KUU)) + a
                 # cvx
                 x = cvxpy.Variable(U_len)
-                objective = cvxpy.Minimize(0.5 * cvxpy.quad_form(x, P) + q.T * x)
+                objective = cvxpy.Minimize(0.5 * cvxpy.quad_form(x, P) + q.T @ x)
                 constraints = [0 <= x, x <= 1]
                 prob = cvxpy.Problem(objective, constraints)
                 # The optimal objective value is returned by `prob.solve()`.
-                result = prob.solve(solver=cvxpy.OSQP if qp_solver == 'OSQP' else cvxpy.ECOS)
+                try:
+                    result = prob.solve(solver=cvxpy.OSQP if qp_solver == 'OSQP' else cvxpy.ECOS)
+                except cvxpy.error.DCPError:
+                    result = prob.solve(solver=cvxpy.OSQP if qp_solver == 'OSQP' else cvxpy.ECOS, gp=True)
 
             # The optimal value for x is stored in `x.value`.
             # print(x.value)
@@ -1670,7 +1723,7 @@ class QueryInstanceLAL(BaseIndexQuery):
 
         print('Building lal regression model from ' + file_path)
         lalModel1 = RandomForestRegressor(n_estimators=parameters['est'], max_depth=parameters['depth'],
-                                          max_features=parameters['feat'], oob_score=True, n_jobs=8)
+                                          max_features=parameters['feat'], oob_score=True, n_jobs=os.cpu_count())
 
         lalModel1.fit(regression_features, np.ravel(regression_labels))
 
@@ -1729,3 +1782,172 @@ class QueryInstanceLAL(BaseIndexQuery):
         # retrieve the real index of the selected datapoint
         selectedIndex = np.asarray(unlabel_index)[selectedIndex1toN]
         return selectedIndex
+
+
+class QueryInstanceCoresetGreedy(BaseIndexQuery):
+    """ICLR 2018 Paper: Active Learning for Convolutional Neural Networks: A Core-Set Approach.
+    The implementation is referred to
+    https://github.com/google/active-learning/blob/master/sampling_methods/kcenter_greedy.py
+
+    Parameters
+    ----------
+    X: 2D array
+        Feature matrix of the whole dataset. It is a reference which will not use additional memory.
+
+    y: array-like
+        Label matrix of the whole dataset. It is a reference which will not use additional memory.
+
+    train_idx: array-like
+        the index of training data.
+
+    distance: str, optional (default='euclidean')
+        Distance metric. Should be one of ['cityblock', 'cosine', 'euclidean', 'l1', 'l2', 'manhattan'].
+    """
+
+    def __init__(self, X, y, train_idx, distance='euclidean'):
+        self.X = X[train_idx,:]
+        self.y = y[train_idx]
+        self.name = 'kcenter'
+        self.features = self.X
+        self.metric = distance
+        self.min_distances = None
+        self.n_obs = self.X.shape[0]
+        self.already_selected = []
+        self.train_idx = train_idx
+
+    def update_distances(self, cluster_centers, only_new=True, reset_dist=False):
+        """Update min distances given cluster centers.
+
+        Args:
+          cluster_centers: indices of cluster centers
+          only_new: only calculate distance for newly selected points and update
+            min_distances.
+          rest_dist: whether to reset min_distances.
+        """
+        if reset_dist:
+            self.min_distances = None
+        if only_new:
+            cluster_centers = [d for d in cluster_centers
+                               if d not in self.already_selected]
+        if cluster_centers:
+            # Update min_distances for all examples given new cluster center.
+            x = self.features[cluster_centers,:]
+            dist = pairwise_distances(self.features, x, metric=self.metric)
+
+            if self.min_distances is None:
+                self.min_distances = np.min(dist, axis=1).reshape(-1, 1)
+            else:
+                self.min_distances = np.minimum(self.min_distances, dist)
+
+    def select(self, label_index, unlabel_index, batch_size=1, **kwargs):
+        """
+        Diversity promoting active learning method that greedily forms a batch
+        to minimize the maximum distance to a cluster center among all unlabeled
+        datapoints.
+
+        Parameters
+        ----------
+        label_index: {list, np.ndarray, IndexCollection}
+            The indexes of labeled samples.
+
+        unlabel_index: {list, np.ndarray, IndexCollection}
+            The indexes of unlabeled samples.
+
+        model: object, optional (default=None)
+            Current classification model, should have the 'predict_proba' method for probabilistic output.
+            If not provided, LogisticRegression with default parameters implemented by sklearn will be used.
+
+        batch_size: int, optional (default=1)
+            Selection batch size.
+
+        Returns
+        -------
+          indices of points selected to minimize distance to cluster centers
+        """
+        already_selected = [np.where(self.train_idx == id)[0].item() for id in label_index]
+        self.update_distances(already_selected, only_new=True, reset_dist=False)
+
+        new_batch = []
+        self.min_distances[already_selected] = -100.0
+        self.already_selected = already_selected
+
+        for _ in range(batch_size):
+            ind = np.argmax(self.min_distances)
+            # New examples should not be in already selected since those points
+            # should have min_distance of zero to a cluster center.
+            assert ind not in self.already_selected
+            assert self.train_idx[ind] in unlabel_index
+
+            self.update_distances([ind], only_new=True, reset_dist=False)
+            new_batch.append(self.train_idx[ind])
+        return new_batch
+
+
+class QueryInstanceDensityWeighted(BaseIndexQuery):
+    """A method that combines uncertainty and diversity in active selection.
+    Reference paper: Curious Machines: Active Learning with Structured Instances.
+
+    Parameters
+    ----------
+    X: 2D array
+        Feature matrix of the whole dataset. It is a reference which will not use additional memory.
+
+    y: array-like
+        Label matrix of the whole dataset. It is a reference which will not use additional memory.
+
+    uncertainty_meansure: str,
+        ['least_confident', 'margin', 'entropy'].
+
+    distance: str, optional (default='euclidean')
+        Distance metric. Should be one of ['cityblock', 'cosine', 'euclidean', 'l1', 'l2', 'manhattan'].
+
+    beta: float, optional (default=1.0)
+        The tradeoff parameter. (The exponent of diversity term)
+
+    """
+    def __init__(self, X, y, uncertainty_meansure="entropy", distance="euclidean", beta=1.0):
+        assert uncertainty_meansure in ['least_confident', 'margin', 'entropy']
+        assert distance in ['cityblock', 'cosine', 'euclidean', 'l1', 'l2', 'manhattan']
+        self.X = X
+        self.y = y
+        self.uncertainty_meansure = uncertainty_meansure
+        self.distance = distance
+        self.beta = beta
+
+    def select(self, label_index, unlabel_index, batch_size=1, model=None, proba_prediction=None, **kwargs):
+        unlab_fea = self.X[unlabel_index]
+        if model is None and proba_prediction is None:
+            raise ValueError("must provide one of model and proba_prediction.")
+        if model is not None:
+            pv = model.predict_proba(unlab_fea)
+        else:
+            assert len(proba_prediction) == len(unlabel_index)
+            pv = np.asarray(proba_prediction)
+
+        # informativeness
+        spv = np.shape(pv)  # shape of predict value
+        if self.uncertainty_meansure == 'entropy':
+            # calc entropy
+            pat = [-np.sum(vec * np.log(vec+1e-9)) for vec in pv]
+            # return unlabel_index[nlargestarg(entro, batch_size)]
+        elif self.uncertainty_meansure == 'margin':
+            # calc margin
+            pat = np.partition(pv, (spv[1] - 2, spv[1] - 1), axis=1)
+            pat = pat[:, spv[1] - 2] - pat[:, spv[1] - 1]
+            # return unlabel_index[nlargestarg(pat, batch_size)]
+        elif self.uncertainty_meansure == 'least_confident':
+            # calc least_confident
+            pat = np.partition(pv, spv[1] - 1, axis=1)
+            pat = 1 - pat[:, spv[1] - 1]
+            # return unlabel_index[nlargestarg(pat, batch_size)]
+        else:
+            raise ValueError("uncertainty_meansure should in ['least_confident', 'margin', 'entropy']")
+
+        # repre
+        dis_mat = pairwise_distances(X=unlab_fea, metric=self.distance)
+        div = np.mean(dis_mat, axis=0)
+        div = div**self.beta
+
+        assert len(pat) == len(div) == len(unlabel_index)
+        scores = np.multiply(pat, div)
+        return np.asarray(unlabel_index)[nlargestarg(scores, batch_size)]
